#
# Copyright (C) 2015-2023 Sergey Malinin
# GPL 3.0 license <http://www.gnu.org/licenses/>
#

from backtrader_next import Indicator

# The modules below should/must define __all__ with the Indicator objects
# of prepend an "_" (underscore) to private classes/variables

from .correlation_trend import *
from .correlation_trend_ssf import *
from .cybernetic_oscillator import *
from .dsma import *
from .ema import *
#from .instantaneous_trend import *
from .laguerre_filter import *
from .laguerre_oscillator import *
from .macd import *
from .mama_fama import *
from .mesa_stochastic import *
from .momentum import *
from .roc import *
from .roofing_filter import *
from .second_derivative import *
from .sma import *
from .supersmoother import *
from .ultimate_smoother import *
from .ultimate_channel import *
from .ultimate_bands import *
from .ultimate_oscillator import *

from .bandpass import *
from .trunc_bandpass import *
